local media = LibStub("LibSharedMedia-3.0")
local PWB = Protwarbar:getPWB()

function Protwarbar:BuildGui()
	-- Ragebar frame setup
	RageFrame = CreateFrame("Frame", nil, UIParent)
	RageFrame:SetFrameStrata("BACKGROUND")
	RageFrame:SetBackdrop({bgFile = media:Fetch("background", "Blizzard Tooltip"), tile = false})
	RageFrame:SetPoint("BOTTOMLEFT", PWB.db.profile.rageFrame.x, PWB.db.profile.rageFrame.y)

	-- Ragebar frame movement
	RageFrame:RegisterForDrag('LeftButton')
	RageFrame:SetClampedToScreen(true)
	RageFrame:SetScript("OnDragStart", dragstart)
	RageFrame:SetScript('OnDragStop', dragstop)

	-- Ragebar setup
	RageBar = CreateFrame("StatusBar",nil,RageFrame)
	RageBar:SetOrientation(PWB.db.profile.rageFrame.orientation)
	RageBar:SetMinMaxValues(0, UnitPowerMax("player"))
	RageBar:SetValue(UnitPower("player"))
	RageBar:SetAllPoints(RageFrame)

	-- Ragebar text setup
	RageBarText = RageBar:CreateFontString(nil, 'OVERLAY')
	RageBarText:SetPoint("CENTER", RageFrame, 0, 0);
	RageBarText:SetFont(media:Fetch("font", PWB.db.profile.rageFrame.font), PWB.db.profile.rageFrame.fontsize, "OUTLINE")
	RageBarText:SetText(UnitPower("player"))


	ShieldFrame = CreateFrame("Frame", nil, UIParent)
	ShieldFrame:SetFrameStrata("BACKGROUND")
	ShieldFrame:SetBackdrop({bgFile = media:Fetch("background", "Blizzard Tooltip"), tile = false})
	ShieldFrame:SetPoint("BOTTOMLEFT", PWB.db.profile.shieldFrame.x, PWB.db.profile.shieldFrame.y)

	-- Shield frame movement
	ShieldFrame:RegisterForDrag('LeftButton')
	ShieldFrame:SetClampedToScreen(true)
	ShieldFrame:SetScript("OnDragStart", dragstart)
	ShieldFrame:SetScript('OnDragStop', dragstop)


	ShieldInfoFrame = CreateFrame("Frame", nil, ShieldFrame)
	ShieldInfoFrame:SetFrameStrata("BACKGROUND")
	ShieldInfoFrame:SetBackdrop({bgFile = media:Fetch("background", "Blizzard Tooltip"), tile = false})
	ShieldInfoFrame:SetPoint("RIGHT", 40, 0)

	ShieldInfoFrame:SetWidth(40)
	ShieldInfoFrame:SetHeight(PWB.db.profile.shieldFrame.height * PWB.db.profile.shieldFrame.barcount)
	ShieldInfoFrame:SetBackdropColor(0.1, PWB.db.profile.shieldFrame.background[2],
		PWB.db.profile.shieldFrame.background[3], 0)

	PWB.shieldBars = {}
	PWB.shieldBarText = {}
	for i = 1, PWB.db.profile.shieldFrame.barcount do
		PWB.shieldBars[i] = Protwarbar:createShieldBar()
		PWB.shieldBarText[i] = Protwarbar:createShieldBarText()
	end

	-- Create Icon frames
	PWB.iconFrames = {}
	PWB.IconCooldown = {}
	for i = 1, PWB_ICON_COUNT do
		PWB.iconFrames[i] = Protwarbar:createIcon(PWB.db.profile.icons[i])
		PWB.IconCooldown[i] = CreateFrame("Cooldown", nil ,PWB.iconFrames[i])
		PWB.IconCooldown[i]:SetAllPoints(PWB.iconFrames[i])
	end

	--create debuff frames
	PWB.debufFrames = {}
	PWB.debufCooldown = {}
	for i = 1, PWB_DEBUF_COUNT do
		PWB.debufFrames[i] = Protwarbar:createIcon(PWB.db.profile.debufs[i])
		PWB.debufCooldown[i] = CreateFrame("Cooldown", nil ,PWB.debufFrames[i])
		PWB.debufCooldown[i]:SetAllPoints(PWB.debufFrames[i])
	end


	-- Information frame setup
	InformationFrame = CreateFrame("Frame", nil, UIParent)
	InformationFrame:SetFrameStrata("BACKGROUND")
	InformationFrame:SetPoint("BOTTOMLEFT", PWB.db.profile.InformationFrame.x, PWB.db.profile.InformationFrame.y)
	InformationFrame:SetBackdrop({bgFile = media:Fetch("background", "Blizzard Tooltip"), tile = false})

	-- Information frame movement
	InformationFrame:RegisterForDrag('LeftButton')
	InformationFrame:SetClampedToScreen(true)
	InformationFrame:SetScript("OnDragStart", dragstart)
	InformationFrame:SetScript('OnDragStop', dragstop)

	-- Information text
	VengeanceText = InformationFrame:CreateFontString(nil, 'OVERLAY')
	VengeanceText:SetPoint("LEFT", 4, 0);
	VengeanceText:SetFont(media:Fetch("font", PWB.db.profile.InformationFrame.font), PWB.db.profile.InformationFrame.fontsize, "OUTLINE")
	VengeanceText:SetText(0)

	Protwarbar:UpdateStyling()
	Protwarbar:UpdateVisiblity()
end

function Protwarbar:createIcon(icon, spellid)
	frame = CreateFrame("Frame", nil, UIParent)
	frame:SetFrameStrata("BACKGROUND")
	frame:SetPoint("BOTTOMLEFT", icon.x, icon.y)

	-- Icon frame movement
	frame:RegisterForDrag('LeftButton')
	frame:SetClampedToScreen(true)
	frame:SetScript("OnDragStart", dragstart)
	frame:SetScript('OnDragStop', dragstop)

	-- Icon texture
	frame.texture = frame:CreateTexture(nil, "BACKGROUND")
	frame.texture:SetPoint("CENTER")
	frame.texture:SetTexture(GetSpellTexture(icon.spellid))

	return frame
end

function Protwarbar:createShieldBar()
	frame = CreateFrame("StatusBar", nil, ShieldFrame)
	frame:SetMinMaxValues(0, 100)
	frame:SetValue(0)

	return frame	
end

function Protwarbar:createShieldBarText()
	frame = RageBar:CreateFontString(nil, 'OVERLAY')

	return frame	
end


function Protwarbar:UpdateStyling()
	RageFrame:SetWidth(PWB.db.profile.rageFrame.width)
	RageFrame:SetHeight(PWB.db.profile.rageFrame.height)
	RageFrame:SetMovable(not PWB.db.profile.locked)
	RageFrame:SetBackdropColor(PWB.db.profile.rageFrame.background[1], PWB.db.profile.rageFrame.background[2],
		PWB.db.profile.rageFrame.background[3], PWB.db.profile.rageFrame.background[4])
	RageFrame:EnableMouse(not PWB.db.profile.locked)
	RageFrame:SetPoint("BOTTOMLEFT", PWB.db.profile.rageFrame.x, PWB.db.profile.rageFrame.y)

	RageBar:SetStatusBarTexture(media:Fetch("statusbar", PWB.db.profile.rageFrame.texture))
	RageBar:SetOrientation(PWB.db.profile.rageFrame.orientation)
	RageBar:SetMinMaxValues(0, UnitPowerMax("player"))
	RageBar:SetReverseFill(PWB.db.profile.rageFrame.reverse)
	RageBarText:SetFont(media:Fetch("font", PWB.db.profile.rageFrame.font), PWB.db.profile.rageFrame.fontsize, "OUTLINE")

	ShieldFrame:SetWidth(PWB.db.profile.shieldFrame.width)
	ShieldFrame:SetHeight(PWB.db.profile.shieldFrame.height * PWB.db.profile.shieldFrame.barcount)
	ShieldFrame:SetMovable(not PWB.db.profile.locked)
	ShieldFrame:SetBackdropColor(PWB.db.profile.shieldFrame.background[1], PWB.db.profile.shieldFrame.background[2],
		PWB.db.profile.shieldFrame.background[3], PWB.db.profile.shieldFrame.background[4])
	ShieldFrame:EnableMouse(not PWB.db.profile.locked)
	ShieldFrame:SetPoint("BOTTOMLEFT", PWB.db.profile.shieldFrame.x, PWB.db.profile.shieldFrame.y)

	-- update styling for shieldbars
	for i = 1, PWB.db.profile.shieldFrame.barcount do
		PWB.shieldBars[i]:SetStatusBarTexture(media:Fetch("statusbar", PWB.db.profile.shieldFrame.texture))
		PWB.shieldBars[i]:SetOrientation(PWB.db.profile.shieldFrame.orientation)
		PWB.shieldBars[i]:SetReverseFill(PWB.db.profile.shieldFrame.reverse)
		PWB.shieldBars[i]:SetWidth(PWB.db.profile.shieldFrame.width)
		PWB.shieldBars[i]:SetHeight(PWB.db.profile.shieldFrame.height)
		PWB.shieldBars[i]:SetPoint("BOTTOMLEFT", 0, (i - 1) * PWB.db.profile.shieldFrame.height)

		PWB.shieldBarText[i]:SetPoint("BOTTOMLEFT", ShieldInfoFrame, 0, (i - 1) * PWB.db.profile.shieldFrame.height)
		PWB.shieldBarText[i]:SetFont(media:Fetch("font", PWB.db.profile.shieldFrame.infoFrame.font), PWB.db.profile.shieldFrame.infoFrame.fontsize, "OUTLINE")
	end

	-- update styling for icons
	for i = 1, PWB_ICON_COUNT do
		PWB.iconFrames[i]:SetWidth(PWB.db.profile.icons[i].size)
		PWB.iconFrames[i]:SetHeight(PWB.db.profile.icons[i].size)
		PWB.iconFrames[i]:SetMovable(not PWB.db.profile.locked)
		PWB.iconFrames[i]:EnableMouse(not PWB.db.profile.locked)
		PWB.iconFrames[i]:SetPoint("BOTTOMLEFT", PWB.db.profile.icons[i].x, PWB.db.profile.icons[i].y)
		PWB.iconFrames[i]:SetBackdrop({bgFile = media:Fetch("background", "None"), 
													edgeFile = media:Fetch("border", PWB.db.profile.icons[i].border), 
				                                    tile = false, edgeSize = PWB.db.profile.icons[i].edgeSize})
		PWB.iconFrames[i].texture:SetWidth(PWB.db.profile.icons[i].textureSize or PWB.db.profile.icons[i].size)
		PWB.iconFrames[i].texture:SetHeight(PWB.db.profile.icons[i].textureSize or PWB.db.profile.icons[i].size)

	end

	-- update styling for debufs
	for i = 1, PWB_DEBUF_COUNT do
		PWB.debufFrames[i]:SetWidth(PWB.db.profile.debufs[i].size)
		PWB.debufFrames[i]:SetHeight(PWB.db.profile.debufs[i].size)
		PWB.debufFrames[i]:SetMovable(not PWB.db.profile.locked)
		PWB.debufFrames[i]:EnableMouse(not PWB.db.profile.locked)
		PWB.debufFrames[i]:SetPoint("BOTTOMLEFT", PWB.db.profile.debufs[i].x, PWB.db.profile.debufs[i].y)
		PWB.debufFrames[i]:SetBackdrop({bgFile = media:Fetch("background", "None"), 
													edgeFile = media:Fetch("border", PWB.db.profile.debufs[i].border), 
				                                    tile = false, edgeSize = PWB.db.profile.debufs[i].edgeSize})
		PWB.debufFrames[i].texture:SetWidth(PWB.db.profile.debufs[i].textureSize or PWB.db.profile.debufs[i].size)
		PWB.debufFrames[i].texture:SetHeight(PWB.db.profile.debufs[i].textureSize or PWB.db.profile.debufs[i].size)
	end	

	InformationFrame:SetWidth(PWB.db.profile.InformationFrame.width)
	InformationFrame:SetHeight(PWB.db.profile.InformationFrame.height)
	InformationFrame:SetMovable(not PWB.db.profile.locked)  
	InformationFrame:EnableMouse(not PWB.db.profile.locked)
	InformationFrame:SetPoint("BOTTOMLEFT", PWB.db.profile.InformationFrame.x, PWB.db.profile.InformationFrame.y)
	VengeanceText:SetFont(media:Fetch("font", PWB.db.profile.InformationFrame.font), PWB.db.profile.InformationFrame.fontsize, "OUTLINE")


	if PWB.db.profile.locked then
		PWB.combat = false -- disable fake combat
		for i = 1, PWB_ICON_COUNT do
			PWB.iconFrames[i]:SetBackdropColor(1, 1, 1, 0)
		end
		for i = 1, PWB_DEBUF_COUNT do
			PWB.debufFrames[i]:SetBackdropColor(1, 1, 1, 0)
		end

		InformationFrame:SetBackdropColor(1, 1, 1, 0)
	else
		PWB.combat = true -- enable fake combat to force display UI
		for i = 1, PWB_ICON_COUNT do
			PWB.iconFrames[i]:SetBackdropColor(1, 1, 1, 1)
		end
		for i = 1, PWB_DEBUF_COUNT do
			PWB.debufFrames[i]:SetBackdropColor(1, 1, 1, 1)
		end

		InformationFrame:SetBackdropColor(1, 1, 1, 1)
		ShieldFrame:SetBackdropColor(1, 1, 1, 1)
	end

end

function Protwarbar:UpdateVisiblity()
	local hideAll = false

	-- if hideooc is enabled and we are not in combat
	if PWB.db.profile.hideooc and not PWB.combat then
		hideAll = true
	end

	if PWB.spec ~= 3 and PWB.db.profile.showInProtOnly then
		hideAll = true
	end

	if PWB.db.profile.enabled == false then
		hideAll = true
	end

	if hideAll then
		InformationFrame:Hide()
		ShieldFrame:Hide()
		ShieldInfoFrame:Hide()
		RageFrame:Hide()
		for i = 1, PWB_ICON_COUNT do
			PWB.iconFrames[i]:Hide()
		end

		for i = 1, PWB_DEBUF_COUNT do
			PWB.debufFrames[i]:Hide()
		end		
	else

		RageFrame:Show()

		-- Hide or show vengeance | Only do so in prot spec
		if PWB.db.profile.InformationFrame.enabled and PWB.spec == 3 then
			InformationFrame:Show()		
		else
			InformationFrame:Hide()
		end

		-- Hide or show the helper icons
		if PWB.db.profile.IconsEnabled then
			for i = 1, PWB_ICON_COUNT do

				-- if the icon is enabled
				if PWB.db.profile.icons[i].enabled then

					-- if the spell is depending on a talent
					if PWB.db.profile.icons[i].talent == nil then
						PWB.iconFrames[i]:Show()
					else
						-- if the talent is active
						if PWB.talents[PWB.db.profile.icons[i].talent] then
							PWB.iconFrames[i]:Show()
						else
							PWB.iconFrames[i]:Hide()
						end
					end
				else
					PWB.iconFrames[i]:Hide()
				end
			end

			for i = 1, PWB_DEBUF_COUNT do
				if PWB.db.profile.debufs[i].enabled then
					PWB.debufFrames[i]:Show()
				else
					PWB.debufFrames[i]:Hide()
				end
			end
		else
			for i = 1, PWB_ICON_COUNT do
				PWB.iconFrames[i]:Hide()
			end
			for i = 1, PWB_DEBUF_COUNT do
				PWB.debufFrames[i]:Hide()
			end
		end

		-- hide or show the shieldbar frames
		if PWB.db.profile.shieldFrame.enabled then
			ShieldFrame:Show()
		else
			for i = 1, PWB.db.profile.shieldFrame.barcount do
				PWB.shieldBars[i]:SetValue(0)
			end
			ShieldFrame:Hide()
		end

		if PWB.db.profile.shieldFrame.infoFrame.enabled then
			ShieldInfoFrame:Show()
		else
			for i = 1, PWB.db.profile.shieldFrame.barcount do
				PWB.shieldBarText[i]:SetText("")
			end
			ShieldInfoFrame:Hide()
		end

	end

end

function dragstart(frame)
	if not PWB.db.profile.locked then
		frame:StartMoving()
	end
end

function dragstop(frame)
	frame:StopMovingOrSizing()

	PWB.db.profile.rageFrame.x = RageFrame:GetLeft()
	PWB.db.profile.rageFrame.y = RageFrame:GetBottom()

	PWB.db.profile.shieldFrame.x = ShieldFrame:GetLeft()
	PWB.db.profile.shieldFrame.y = ShieldFrame:GetBottom()

	for i = 1, PWB_ICON_COUNT do
		PWB.db.profile.icons[i].x = PWB.iconFrames[i]:GetLeft()
		PWB.db.profile.icons[i].y = PWB.iconFrames[i]:GetBottom()
	end

	for i = 1, PWB_DEBUF_COUNT do
		PWB.db.profile.debufs[i].x = PWB.debufFrames[i]:GetLeft()
		PWB.db.profile.debufs[i].y = PWB.debufFrames[i]:GetBottom()
	end

	PWB.db.profile.InformationFrame.x = InformationFrame:GetLeft()
	PWB.db.profile.InformationFrame.y = InformationFrame:GetBottom()

end